header {
package com.humanfactor.rw.labelbuilder.parser;
import org.apache.log4j.Logger;
import java.io.*;
}

/** A parser used to break up a single template into chunks, text literals
 *  and attribute expressions.
 */
class TemplateParser extends Parser;

options {
    k=2;
    buildAST=true;
}

{
private static Logger logger = Logger.getLogger(TemplateParser.class);

protected LabelTemplate self;

public void reportError(RecognitionException e) {
	logger.error("template parser error", e);
}
}

template[LabelTemplate self]
{
	this.self = self;
}
    :   (   s:LITERAL^  {self.addChunk(new StringRef(self,s.getText()));}
        |   template_variable 
        )*
    ;
    
template_variable
	:	LBRACE^ (LITERAL|template_variable)+ RBRACE
	;
	
template_expression
	{ System.out.println("Parsing template_expression LA(1): '" + tokenNames[LA(1)]+"' LA(2): '" + tokenNames[LA(2)] + "'"); }
	:	( 
			i1:identifier QUESTION^ choice_stmt 
		|	i2:identifier COMMA^ formatting_spec
		|	i3:identifier 
		)
	;
	
identifier
	: i:IDENT { System.out.println("Parsed identifier: "+i.getText()); }
	;
	
formatting_spec
	:	data_type (COMMA format_style)?;
	
data_type
	:	("NUMBER"|"DATE"|"TIME");
	
format_style
	: (STRING_LITERAL|"LONG"|"MEDIUM"|"SHORT");
	
choice_stmt
	:	(boolean_choice_stmt | integer_range_choice_stmt | enumeration_choice_stmt);

boolean_choice_stmt
	:	(boolean_option ( OR boolean_option )+);
	
integer_range_choice_stmt
	:	(integer_range_option ( OR integer_range_option )+);
	
enumeration_choice_stmt
	:	(enumeration_option (OR enumeration_option )+);
	
boolean_option
	: string_producer;
	
integer_range_option
	:	INT COLON string_producer;
	
enumeration_option
	:	IDENT COLON string_producer;
	
string_producer
	: ( STRING_LITERAL | template_variable );
	
integer_literal
 	:   INT
    ;

string_literal
	:  (STRING_LITERAL)+
	;

character_literal
	:   CHAR_LITERAL
	;

floating_pt_literal
	:   f:FLOAT
     	;
    
    
/** Break up an input text stream into chunks of either plain text
 *  or template actions in "$...$".  Treat IF and ENDIF tokens
 *  specially.
 */
class TemplateLexer extends Lexer;

options {
    k=4; // see "$endif$"
    charVocabulary = '\u0001'..'\uFFFE';
}

{
private static Logger logger = Logger.getLogger(TemplateLexer.class);

public void reportError(RecognitionException e) {
	logger.error("template lexer error", e);
}
}

LITERAL
	{ System.out.println("Calling LITERAL: LA(1): "+LA(1)+" LA(2): "+ LA(2)); }
    :   {LA(1)!='\r'&&LA(1)!='\n'}?
        ( options { generateAmbigWarnings=false;}
          {
          int loopStartIndex=text.length();
          int col=getColumn();
          }
        : '\\'! '{'  // allow escaped delimiter
        | '\\'! '}'  // allow escaped delimiter
        | '\\' (~('{'|'}'))  // otherwise ignore escape char
        | ~('{'|'\r'|'\n'|'}')
        )+
        { System.out.println("Found LITERAL: "+getText()); }
    ;
    


QUESTION
options {
  paraphrase = "?";
}
	:	'?'
	;

LBRACE
options {
  paraphrase = "{";
}
	:	'{' { System.out.println("Found LBRACE: "+getText()); }
	;

RBRACE
options {
  paraphrase = "}";
}
	:	'}' { System.out.println("Found RBRACE: "+getText()); }
	;

COLON
options {
  paraphrase = ":";
}
	:	':'
	;

COMMA
options {
  paraphrase = ",";
}
	:	','
	;
	
OR
options {
  paraphrase = "|";
}
	:	'|'
	;

CHAR_LITERAL 	
options {
  paraphrase = "a character literal";
}
	:
	'\'' 
	( ESC | ~'\'' ) 
	'\''
	;

STRING_LITERAL 	
options {
  paraphrase = "a string literal";
}
	:
	'"' 
	(ESC|~'"')* 
	'"'
	;


protected
ESC	
options {
  paraphrase = "an escape sequence";
}
	:	'\\' ('n'|'t'|'v'|'b'|'r'|'f'|'a'|'\\'|'?'|'\''|'"')
	;

protected
VOCAB
options {
  paraphrase = "an escaped character value";
}
	:	'\3'..'\377'
	;

protected
DIGIT
options {
  paraphrase = "a digit";
}
	:	'0'..'9'
	;

INT	
options {
  paraphrase = "an integer value";
}
	:    (DIGIT)+                  // base-10 
             (  '.' (DIGIT)*                      	{$setType(FLOAT);}
	         (('e' | 'E') ('+' | '-')? (DIGIT)+)? 
	     |   ('e' | 'E') ('+' | '-')? (DIGIT)+   	{$setType(FLOAT);}
             )?
	;

FLOAT
options {
  paraphrase = "an floating point value";
}

	:    '.' (DIGIT)+ (('e' | 'E') ('+' | '-')? (DIGIT)+)?
     	;
     	
IDENTIFIER
    :   LETTER (LETTER|JAVA_ID_DIGIT)*
    ;

/**I found this char range in JavaCC's grammar, but Letter and Digit overlap.
   Still works, but...
 */
LETTER
    :  '\u0024' |
       '\u0041'..'\u005a' |
       '\u005f' |
       '\u0061'..'\u007a' |
       '\u00c0'..'\u00d6' |
       '\u00d8'..'\u00f6' |
       '\u00f8'..'\u00ff' |
       '\u0100'..'\u1fff' |
       '\u3040'..'\u318f' |
       '\u3300'..'\u337f' |
       '\u3400'..'\u3d2d' |
       '\u4e00'..'\u9fff' |
       '\uf900'..'\ufaff'
    ;

JAVA_ID_DIGIT
    :  '\u0030'..'\u0039' |
       '\u0660'..'\u0669' |
       '\u06f0'..'\u06f9' |
       '\u0966'..'\u096f' |
       '\u09e6'..'\u09ef' |
       '\u0a66'..'\u0a6f' |
       '\u0ae6'..'\u0aef' |
       '\u0b66'..'\u0b6f' |
       '\u0be7'..'\u0bef' |
       '\u0c66'..'\u0c6f' |
       '\u0ce6'..'\u0cef' |
       '\u0d66'..'\u0d6f' |
       '\u0e50'..'\u0e59' |
       '\u0ed0'..'\u0ed9' |
       '\u1040'..'\u1049'
   ;

IDENT
options {
  testLiterals = true;
  paraphrase = "an identifer";
}

	:	('a'..'z'|'A'..'Z') ('a'..'z'|'A'..'Z'|'_'|'0'..'9')*
	;
	
{import java.lang.Math;}
class TemplateTreeWalker extends TreeParser;

template returns [String r]
  { String a;
  	r = ""; }

  : #(LITERAL a=template)  { r=r+a; }
  | #(LBRACE a=template) { r=r+a; }
  ;
	
    