header {
package com.humanfactor.rw.labelbuilder.parser;
import org.apache.log4j.Logger;
import java.io.*;
}

/** A parser used to break up a single template into chunks, text literals
 *  and attribute expressions.
 */
class TemplateParser extends Parser;

options {
    k=2;
    buildAST=true;
}

tokens {
	CHOICE_EXPRESSION;
	FORMATTED_EXPRESSION;
	PLAIN_EXPRESSION;
	BOOLEAN_CHOICE;
	ENUMERATION_CHOICE;
	INTEGER_RANGE_CHOICE;
	DATE_FORMATTING_SPEC;
	TIME_FORMATTING_SPEC;
	NUMBER_FORMATTING_SPEC;
	SUMMATION_LIST;
}

{
private static Logger logger = Logger.getLogger(TemplateParser.class);
}

template
    :   template_part (PLUS! template_part)* //{#template = #(#[SUMMATION_LIST],#template);}
    ;
    
template_part
	:	( STRING_LITERAL | template_variable )
	;
    
template_variable
	:	LBRACE! template_expression RBRACE!
	;
	
template_expression
	:	( 
			IDENT QUESTION! choice_list 
				{#template_expression = #([CHOICE_EXPRESSION, "CHOICE_EXPRESSION"], #template_expression); }
		|	IDENT COMMA! formatting_spec
				{#template_expression = #([FORMATTED_EXPRESSION, "FORMATTED_EXPRESSION"], #template_expression); }
		|	IDENT
				{#template_expression = #([PLAIN_EXPRESSION, "PLAIN_EXPRESSION"], #template_expression); }
		)
	;
	
identifier
	: i:IDENT { logger.debug("Parsed identifier: " + i.getText()); }
	;
	
formatting_spec
	:	DATE (COMMA! date_time_format_pattern)?
			{#formatting_spec = #([DATE_FORMATTING_SPEC, "DATE_FORMATTING_SPEC"], #formatting_spec);}
	|	TIME (COMMA! date_time_format_pattern)?
			{#formatting_spec = #([TIME_FORMATTING_SPEC, "TIME_FORMATTING_SPEC"], #formatting_spec);}
	| 	NUMBER (COMMA! number_format_pattern)?
			{#formatting_spec = #([NUMBER_FORMATTING_SPEC, "NUMBER_FORMATTING_SPEC"], #formatting_spec);}
	;
	
date_time_format_pattern
	: (STRING_LITERAL|SHORT|MEDIUM|LONG|FULL)
	;
	
number_format_pattern
	: (STRING_LITERAL|INTEGER|CURRENCY|PERCENT)
	;
	
// {#choice_stmt = #([BOOLEAN_CHOICE],#boolean_choice_stmt); #boolean_choice_stmt.setType(BOOLEAN_CHOICE);}

//choice_list
//	:	BOOLEAN_CHOICE_LIST
//	|	INTEGER_RANGE_CHOICE_LIST
//	|	ENUMERATION_CHOICE_LIST
//	;

choice_list
	:	boolean_choice_stmt 
	|	integer_range_choice_stmt 
	|	enumeration_choice_stmt 
	;

boolean_choice_stmt
	:	(boolean_option OR! boolean_option) 
			
	;
	
integer_range_choice_stmt
	:	(integer_range_option (OR! integer_range_option)+)
			
	;
	
enumeration_choice_stmt
	:	(enumeration_option (OR! enumeration_option)+)
			
	;
	
boolean_option
	: 	string_producer
			{#boolean_option = #([BOOLEAN_CHOICE, "BOOLEAN_CHOICE"], #boolean_option);}
	;
	
integer_range_option
	:	INT COLON! string_producer
			{#integer_range_option = #([INTEGER_RANGE_CHOICE, "INTEGER_RANGE_CHOICE"], #integer_range_option);}
	;
	
enumeration_option
	:	IDENT COLON! string_producer
			{#enumeration_option = #([ENUMERATION_CHOICE, "ENUMERATION_CHOICE"], #enumeration_option);}
	;
	
string_producer
	: ( STRING_LITERAL | template_variable );
	
integer_literal
 	:   INT
    ;

string_literal
	:  STRING_LITERAL
	;

character_literal
	:   CHAR_LITERAL
	;

floating_pt_literal
	:   f:FLOAT
     	;
    
    
/** Break up an input text stream into chunks of either plain text
 *  or template actions in "$...$".  Treat IF and ENDIF tokens
 *  specially.
 */
class TemplateLexer extends Lexer;

options {
    k=5; // see "$endif$"
    charVocabulary = '\u0001'..'\uFFFE';
}

tokens {
	DATE	= "DATE";
	TIME	= "TIME";
	NUMBER	= "NUMBER";
	SHORT	= "SHORT";
	MEDIUM	= "MEDIUM";
	LONG	= "LONG";
	FULL	= "FULL";
	INTEGER	= "INTEGER";
	CURRENCY= "CURRENCY";
	PERCENT	= "PERCENT";
}

{
private static Logger logger = Logger.getLogger(TemplateLexer.class);

public void reportError(RecognitionException e) {
	logger.error("template lexer error", e);
}
}

QUESTION
options {
  paraphrase = "?";
}
	:	'?'
	;

LBRACE
options {
  paraphrase = "{";
}
	:	'{'
	;

RBRACE
options {
  paraphrase = "}";
}
	:	'}'
	;

COLON
options {
  paraphrase = ":";
}
	:	':'
	;

COMMA
options {
  paraphrase = ",";
}
	:	','
	;
	
OR
options {
  paraphrase = "|";
}
	:	'|'
	;
	
PLUS
options {
  paraphrase = "+";
}
	:	'+'
	;

WS	
options {
  paraphrase = "white space";
}
	:	(' '
	|	'\t'
	|	'\n'  { newline(); }
	|	'\r')
		{ $setType(Token.SKIP); }
	;

CHAR_LITERAL 	
options {
  paraphrase = "a character literal";
}
	:
	'\'' 
	( ESC | ~'\'' ) 
	'\''
	;

STRING_LITERAL 	
options {
  paraphrase = "a string literal";
}
	:
	'"' 
	(ESC|~'"')* 
	'"'			{ setText(getText().substring(1, getText().length()-1)); }
	;


protected
ESC	
options {
  paraphrase = "an escape sequence";
}
	:	'\\' ('n'|'t'|'v'|'b'|'r'|'f'|'a'|'\\'|'?'|'\''|'"')
	;

protected
VOCAB
options {
  paraphrase = "an escaped character value";
}
	:	'\3'..'\377'
	;

protected
DIGIT
options {
  paraphrase = "a digit";
}
	:	'0'..'9'
	;

INT	
options {
  paraphrase = "an integer value";
}
	:    (DIGIT)+                  // base-10 
             (  '.' (DIGIT)*                      	{$setType(FLOAT);}
	         (('e' | 'E') ('+' | '-')? (DIGIT)+)? 
	     |   ('e' | 'E') ('+' | '-')? (DIGIT)+   	{$setType(FLOAT);}
             )?
	;

FLOAT
options {
  paraphrase = "an floating point value";
}
	:    '.' (DIGIT)+ (('e' | 'E') ('+' | '-')? (DIGIT)+)?
     	;

IDENT
options {
  testLiterals = true;
  paraphrase = "an identifer";
}

	:	('a'..'z'|'A'..'Z') ('a'..'z'|'A'..'Z'|'_'|'0'..'9')*
	;
	
{
import java.lang.Math;
import com.humanfactor.rw.labelbuilder.choice.BooleanChoiceSelector;
import com.humanfactor.rw.labelbuilder.choice.IntegerRangeChoiceSelector;
import com.humanfactor.rw.labelbuilder.choice.EnumChoiceSelector;
import com.humanfactor.rw.labelbuilder.choice.ChoiceSelector;
import com.humanfactor.rw.labelbuilder.choice.ChoiceSelector.Choice;
import com.humanfactor.rw.labelbuilder.LabelBuilderException;
import com.humanfactor.rw.labelbuilder.resolver.IVariableNameResolver;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.util.Date;
}

class TemplateTreeWalker extends TreeParser("com.humanfactor.rw.labelbuilder.parser.BaseTemplateTreeWalker");

{
	
private static Logger logger = Logger.getLogger(TemplateTreeWalker.class);
public static final String EMPTY_STRING = "";

public TemplateTreeWalker(IVariableNameResolver resolver) {
   super(resolver);
}

}

template returns [StringBuffer r] throws LabelBuilderException
  { String a;
  	r = new StringBuffer(); }

	: (a=template_part { r.append(a); })+	
	;
	
template_part returns [String r] throws LabelBuilderException
	{ String a;
		r = EMPTY_STRING; }
	: s:STRING_LITERAL { r = s.getText(); }
	| a=expression 	{ r = a; }
	;
  
expression returns [String r] throws LabelBuilderException
  { Choice c;
  	Enum e;
  	DateFormat df;
  	NumberFormat nf;
  	String a, b, n;
  	ChoiceSelector choiceSelector = null;
  	r = ""; }

	: #(CHOICE_EXPRESSION a=identifier (c=choice 
		{ 	if (choiceSelector == null) {
				Object variable = nameResolver.resolveVariableName(a);
				choiceSelector = ChoiceSelector.createChoiceSelector(variable);
			}
			choiceSelector.addChoice(c); 
			//logger.debug("CHOICE_EXPRESSION added: selector var=" + a + " choice value=" + c); 
		} )+ 
	  ) 
	  	{	Object variable = nameResolver.resolveVariableName(a);
	  		r = choiceSelector.selectChoice(variable);
	  	}
	  	
  	| #(PLAIN_EXPRESSION a=identifier) 
  			{ r = resolveString(a); }
  			
  	| #(FORMATTED_EXPRESSION a=identifier 
  		(	  df=date_time_formatting_spec
  				{	Date date = resolveDate(a);
  					r = df.format(date);
  				}	
  			| nf=number_formatting_spec
  				{	Number number = resolveNumber(a);
  					r = nf.format(number);
  				}
  		)) 
	;
  
choice returns [Choice c] throws LabelBuilderException
  { String a, b;
  	int i;
  	c = null; }

	: #(ENUMERATION_CHOICE a=identifier b=string_producer)  { c = new Choice(a, b); }
	| #(INTEGER_RANGE_CHOICE i=integer b=string_producer) { c = new Choice(new Integer(i).toString(), b); }
	| #(BOOLEAN_CHOICE a=string_producer) { c = new Choice(null, a); }
	;
	
date_time_formatting_spec returns [DateFormat dateFormat] throws LabelBuilderException
  { String b;
  	Integer f;
  	b = null;
  	f = null;
  	dateFormat = null; }
  	
	:	#(DATE_FORMATTING_SPEC DATE (f=date_time_format_pattern|b=string_literal)?) 
			{ 	if (b != null) {
					dateFormat = getDateFormat(b);
				} else if (f != null) {
					dateFormat = getDateFormat(f);
				} else {
					dateFormat = getDateFormat();
				}
			}
	|	#(TIME_FORMATTING_SPEC TIME (f=date_time_format_pattern|b=string_literal)?) 
			{ 	if (b != null) {
					dateFormat = getTimeFormat(b);
				} else if (f != null) {
					dateFormat = getTimeFormat(f);
				} else {
					dateFormat = getTimeFormat();
				}
			}
	;
	
number_formatting_spec returns [NumberFormat numberFormat] throws LabelBuilderException
  { String b;
  	Integer f;
  	b = null;
  	f = null;
  	numberFormat = null; }
  	
	:	#(NUMBER_FORMATTING_SPEC NUMBER (f=number_format_pattern|b=string_literal)?) 
			{ 	if (b != null) {
					numberFormat = getNumberFormat(b);
				} else if (f != null) {
					numberFormat = getNumberFormat(f);
				} else {
					numberFormat = getNumberFormat();
				}
			}
	;
	
string_producer returns [String r] throws LabelBuilderException
	{ String e;
		r = ""; }
	: s:STRING_LITERAL { r = s.getText(); }
	| e=expression { r = e; }
	;

integer returns [int r]
	{ r = 0; }
	:	i:INT { r = Integer.parseInt(i.getText()); }
	;
	
date_time_format_pattern returns [Integer token]
	{ token = null; }
	:	s:SHORT		{ token = new Integer(s.getType()); }
	|	m:MEDIUM	{ token = new Integer(m.getType()); }
	|	l:LONG		{ token = new Integer(l.getType()); }
	|	f:FULL		{ token = new Integer(f.getType()); }
	;
	
number_format_pattern returns [Integer token]
	{ token = -1; }
	:	INTEGER
	|	CURRENCY
	|	PERCENT
	;
	
string_literal returns [String r]
	{ r = ""; }
	: s:STRING_LITERAL		{ r = s.getText(); }
							//{ s.setText(s.getText().substring(1, s.getText().length()-1));
							//r = s.getText(); logger.debug("Converting string literal"); }
	;	
	
enum_identifier returns [Enum e] throws LabelBuilderException
	{ 	String a;
		e = null; }
	:	a=identifier { e = resolveEnumeration(a); }
	;

identifier returns [String r] throws LabelBuilderException
	{ r = ""; }
	: i:IDENT { r = i.getText(); }
	;
	
//template_variable returns [String r]
//  { String a, b;
//  	r = ""; }
//  	: #(QUESTION i1:IDENT b=option_list) { logger.debug("QUESTION: selector var="+i1+" value="+b); r=r+a; }
//  	| #(COMMA a=template b=template) { r=r+a; }
//  	| i:IDENT { r = i.getText(); }
//  	;
  	